import "Turbine";
import "Turbine.UI";
import "Turbine.UI.Lotro";
import "Turbine.Gameplay";
import "Kiki.SuperChrono.Includes.ChatLogger";
import "Kiki.SuperChrono.Includes.OnScreen";
import "Kiki.SuperChrono.Includes.DateUtils";
import "Kiki.SuperChrono.Includes.NumberUtils";
import "Kiki.SuperChrono.Includes.StringUtils";
import "Kiki.SuperChrono.Windows.Log";
import "Kiki.SuperChrono.Windows.Chrono";
import "Kiki.SuperChrono.Windows.Maj";
import "Kiki.SuperChrono.Windows.Export";
import "Kiki.SuperChrono.UTF";
import "Kiki.SuperChrono.Languages";

--Plugin Starts Here

-- Dump table function for checking tables on the fly ---------------
function dump(o)
    if type(o) == 'table' then
        local s = '{\n'
        for k,v in pairs(o) do
                if type(k) ~= 'number' then k = '"'..k..'"' end
                s = s .. '['..k..'] = ' .. dump(v) .. '\n'
        end
        return s .. '}\n'
    else
        return tostring(o)
    end
end


--This function returns a deep copy of a given table ---------------
function deepcopy(object)
    local lookup_table = {}
    local function _copy(object)
        if type(object) ~= "table" then
            return object
        elseif lookup_table[object] then
            return lookup_table[object]
        end
        local new_table = {}
        lookup_table[object] = new_table
        for index, value in pairs(object) do
            new_table[_copy(index)] = _copy(value)
        end
        return setmetatable(new_table, getmetatable(object))
    end
    return _copy(object)
end


-- This function merges table two into table one, overwriting any matching entries.
function mergeTables(t1, t2)
	for k, v in pairs(t2) do
		if (type(v) == "table") and (type(t1[k] or false) == "table") then
			mergeTables(t1[k], t2[k])
		else
			t1[tostring(k)] = v
		end
	end
	return t1
end

-- Recupere la langue du client
function getClientLanguage()
	
	if Turbine.Shell.IsCommand("hilfe") then
		return "GERMAN";
	elseif Turbine.Shell.IsCommand("aide") then
		return "FRENCH";
	else 
		return "ENGLISH";
	end
end


-- Chargement des settings
function RefactorSettings(settings)
	Turbine.Shell.WriteLine("<rgb=#FFFF00>Refactoring settings</rgb>");
	settings.version = SETTINGSVERSION;
	return settings;
end 

function loadSettings()
	-- Load the settings
	function _gss()
		return Turbine.PluginData.Load(Turbine.DataScope.Account, "KikiSuperChrono");
	end

	local isOk, SavedSettings = pcall(_gss);

	if not isOk then
		-- Loaded with errors
		Turbine.Shell.WriteLine("Error loading settings");
		SavedSttings = nil;
	end

	-- Check the list to see if it's valid.
	if type(SavedSettings) == 'table' then
		if SavedSettings.version == gVersion then
			return SavedSettings;
		else
			return RefactorSettings(SavedSettings);
		end
	else
		return deepcopy(DEFAULT_SETTINGS);
	end

end

-- sauvegarde des settings
function saveSettings(s)
	Turbine.PluginData.Save(Turbine.DataScope.Account, "KikiSuperChrono", s);
	Includes.Trace("Warm data saved");
end

-- Unload du plugin
Turbine.Plugin.Unload = function (sender, args)
	saveSettings(gSettings);
	Includes.Trace("Shutdown completed");
end

-- Default Settings
DEFAULT_SETTINGS =
{
	lang = "FRENCH";
	pluginName="SuperChrono";
	msgColor = "FF0000";
	debug = false;
	log =
	{
		x = 100;
		y = 100;
		visible = true;
	};
	chrono =
	{
		x = 700;
		y = 100;
		visible = true;
	};
	configurations =
	{ 
	};
};


RESOURCESDIR = "Kiki/SuperChrono/Resources/";
SETTINGSVERSION = "1.0";

gSettings = loadSettings();

-- override le nom du plgin
gSettings.pluginName = "SuperChrono";
-- Override la langue
gSettings.lang = getClientLanguage();

-- Charge les traductions
gLang = getLang(gSettings.lang); 

-- Charge le joueur actuel
gPlayer = Turbine.Gameplay.LocalPlayer.GetInstance();
gName = gPlayer:GetName();

-- Affiche la fenetre de logs
Windows.DrawLogWindow();
Windows.DrawChronoWindow();
Windows.wMajDraw();

-- En cas Includes.Trace(hangement de l'etat de combat
gPlayer.InCombatChanged = function(sender, args)
	Windows.SetBarInCombat(gPlayer:IsInCombat());
end

-- Traqueur de messages
Includes.InitiateChatLogger();


-- Si une config est indique dans les settings, on la charge initialement
if gSettings.lastConfigName ~= nil then
	Windows.ChronoActivateConfigurationByName(gSettings.lastConfigName);
end

-- Retrouve la version du plugin
Includes.Trace("loaded (version ".. Plugins["SuperChrono"]:GetVersion() .. " by Kikiadoc@Sirannon)");

-- Includes.Trace(dump(gSettings));













